#!/usr/bin/env python
# coding: utf-8

"""
13.1 NetworkX
(p.199 ~ p.227)
"""

# In[1]: p.199
import networkx as nx
G = nx.Graph()


# In[2]:
G.add_node(1)
print(G.nodes())


# In[3]:
G.add_node('P')
G.add_node('Hi')
print(G.nodes())


# In[4]:
G.add_nodes_from([2, 3])
print(G.nodes())


# In[5]: p.201
nx.draw(G, with_labels=True, node_color='lightblue')


# In[6]:
G.add_edge(1, 2)
nx.draw(G, with_labels=True, node_color='lightblue', edge_color='grey')


# In[7]:
print(G.edges())


# In[8]: p.202
G.add_edges_from([(1,2), (1,3), (1,4), (1,5)])
print(G.edges())


# In[9]: p.203
import networkx as nx
G = nx.path_graph(4)
nx.draw(G, with_labels=True, node_color='lightblue', edge_color='grey')


# In[10]:
import networkx as nx
G = nx.Graph()
nx.add_path(G, [0,1,2,3])
nx.draw(G, with_labels=True, node_color='lightblue', edge_color='grey')


# In[11]: p.204
import networkx as nx
G = nx.Graph()
nx.add_path(G, [0,1,2,3])

print(G.degree(0))  #  0   
print(G.degree([0, 1]))  #  0  1   
print(G.degree())  #  忡   


# In[12]:
print(G.edges())
G.remove_edge(1, 2)
print(G.edges())


# In[13]:
print(G.edges())
G.remove_edges_from([(1, 2), (1, 3)])
print(G.edges())


# In[14]:
print(G.nodes())
G.remove_node(2)
print(G.nodes())


# In[15]: p.205
print(G.nodes())
G.remove_nodes_from([0, 1, 3])
print(G.nodes())


# In[16]:
import networkx as nx
G = nx.Graph()
G.add_nodes_from([1, 2, 3, 4, 5])
G.add_edges_from([(1,2), (1,3), (1,4), (1,5), (4, 5)])

print('No. nodes:', G.number_of_nodes())
print('No. edges:', G.number_of_edges())


# In[17]:
G.remove_edge(1, 3)

print('No. nodes:', G.number_of_nodes())
print('No. edges:', G.number_of_edges())


# In[18]: p.206
import networkx as nx
G = nx.Graph()
G.add_nodes_from([1, 2, 3, 4, 5])
G.add_edges_from([(1,2), (1,3), (1,4), (1,5), (4, 5)])

G.remove_node( 3 )

print('No. nodes:', G.number_of_nodes())
print('No. edges:', G.number_of_edges())


# In[19]: p.207
import networkx as nx
G = nx.Graph()
G.add_edge('a', 'b', weight=3)
G.add_edge('b', 'c', weight=4)
G.add_edge('c', 'd', weight=3)
G.add_edge('d', 'b', weight=8)
G.add_edge('c', 'a', weight=10)
labels=nx.get_edge_attributes(G, 'weight')
print(labels)
nx.draw(G, with_labels=True, node_color='lightblue',
                                            edge_color='grey')


# In[20]: p.208
nx.shortest_path(G, 'a', 'd', weight='weight')


# In[21]:
import networkx as nx
G = nx.Graph()
G.add_edge('a', 'b', weight=4)
G.add_edge('a', 'd', weight=8)
G.add_edge('b', 'd', weight=3)
G.add_edge('b', 'e', weight=10)
G.add_edge('f', 'd', weight=12)
G.add_edge('f', 'e', weight=7)
labels=nx.get_edge_attributes(G, 'weight')
print(labels)
nx.draw(G, with_labels=True, node_color='lightblue',
                                            edge_color='grey')


# In[22]: p.209
import networkx as nx
import matplotlib.pyplot as plt
G = nx.erdos_renyi_graph(25, 0.2)
nx.draw(G)


# In[23]:
G = nx.watts_strogatz_graph(30, 3, 0.1)
nx.draw(G)


# In[24]: p.210
G = nx.barabasi_albert_graph(25, 5)
nx.draw(G)


# In[25]: p.211
G = nx.random_lobster(25, 0.9, 0.9)
nx.draw(G)


# In[26]:
from scipy.stats import bernoulli
for i in range(6):
    print(bernoulli.rvs(p=0.33))


# In[27]: p.212
import numpy as np
import networkx as nx
import matplotlib.pyplot as plt
from scipy.stats import bernoulli
def erdosGraph(N, p):
    G = nx.Graph()
    G.add_nodes_from(range(N))
    listG = list(G.nodes())
    for i, node1 in enumerate(listG):
        for node2 in listG[i+1:]:
            if (bernoulli.rvs(p=p)):
                G.add_edge(node1, node2)
    return G

G1 = erdosGraph(80, 0.3)
plt.hist(list([d for n, d in G1.degree()]), histtype='step')


# In[28]:
import numpy as np
import networkx as nx
import matplotlib.pyplot as plt
from scipy.stats import bernoulli
def erdosGraph(N, p):
    G = nx.Graph()
    G.add_nodes_from(range(N))
    listG = list(G.nodes())
    for i, node1 in enumerate(listG):
        for node2 in listG[i+1:]:
            if (bernoulli.rvs(p=p)):
                G.add_edge(node1, node2)
    return G
def pltGraph(G):
    plt.hist(list([d for n, d in G.degree()]), histtype='step')

G1 = erdosGraph(80, 0.3)
pltGraph(G1)
G2 = erdosGraph(80, 0.3)
pltGraph(G2)
G3 = erdosGraph(80, 0.3)
pltGraph(G3)


# In[29]: p.214
import numpy as np
import networkx as nx
K = np.loadtxt("test.csv", delimiter=",")
G = nx.to_networkx_graph(K)
nx.draw(G, with_labels=True, node_color='yellow', edge_color='red')


# In[30]: p.216
import networkx as nx
G = nx.karate_club_graph()
import matplotlib.pyplot as plt
nx.draw(G, with_labels=True, node_color='lightblue', edge_color='yellow')


# In[31]: p.217
G.degree()


# In[32]:
print(G.degree())


# In[33]:
G.degree(16)


# In[34]: p.218
import numpy as np
import networkx as nx
import matplotlib.pyplot as plt
G = nx.karate_club_graph()

print(' : %d' % G.number_of_nodes())
print(' : %d' % G.number_of_edges())
print(' : %.1f' % np.mean
([d for n, d in G.degree()]))


# In[35]: p.219
import networkx as nx
G = nx.karate_club_graph()
degree = [d for n, d in G.degree()]
nx.draw(G, with_labels=True, node_size=100, node_color='yellow', edge_color='red')


# In[36]: p.220
import networkx as nx
G = nx.karate_club_graph()
degree = [d for n, d in G.degree()]
nx.draw(G, with_labels=True, node_size=[n*100 for n in degree], node_color='yellow', edge_color='red')


# In[37]: p.221
import numpy as np
import networkx as nx
import matplotlib.pyplot as plt
import zipfile
zf = zipfile.ZipFile('football.zip')
txt = zf.read('football.txt').decode()
gml = zf.read('football.gml').decode()
gml = gml.split('\n')[1:]
G = nx.parse_gml(gml)

nx.draw(G, with_labels=True, node_color="yellow", edge_color="green")
plt.savefig('football.pdf') # pdf ϱ


# In[38]: p.223
import numpy as np
import networkx as nx
K = np.loadtxt("social.csv", delimiter=",")
G = nx.to_networkx_graph(K)
nx.draw(G, with_labels=True, node_color='yellow', edge_color='red')


# In[39]: p.
K = np.loadtxt("social.csv", delimiter=",")
G = nx.to_networkx_graph(K)
# ksub = nx.connected_component_subgraphs(G)  # networkx  2.4  
ksub = (G.subgraph(c) for c in nx.connected_components(G))
klist = list(ksub)
print(len(klist))


# In[40]: p.224
import numpy as np
import networkx as nx
K = np.loadtxt("social.csv", delimiter=",")
G = nx.to_networkx_graph(K)
# ksub = nx.connected_component_subgraphs(G)  # networkx  2.4  
ksub = (G.subgraph(c) for c in nx.connected_components(G))
klist = list(ksub)

nx.draw(klist[0], with_labels=True, node_color='yellow', edge_color='red')


# In[41]: p.225
import numpy as np
import networkx as nx
K = np.loadtxt("social.csv", delimiter=",")
G = nx.to_networkx_graph(K)
# maxG = max(nx.connected_component_subgraphs(G), key=len)  # networkx  2.4  
maxG = max((G.subgraph(c) for c in nx.connected_components(G)), key=len)
nx.draw(maxG, with_labels=True, node_color='yellow', edge_color='red')

