#!/usr/bin/env python
# coding: utf-8

"""
16.3 딥러닝 모델의 성능
<3> 학습의 조기종료와 Dropout 기능
p.347 ~ p.353
"""

# In[1]: p.348
from tensorflow.keras import models
from tensorflow.keras import layers
model = models.Sequential()

model.add(layers.Dense(2, input_shape=(4, )))
model.add(layers.Dense(3, activation='softmax'))


# In[2]:
model.compile(optimizer='rmsprop', loss='categorical_crossentropy', metrics=['accuracy'])


# In[3]:
#16.3<2>의 내용 (선행 코드를 위해 넣음)
from sklearn import datasets
iris = datasets.load_iris()
X = iris.data
y = iris.target
from sklearn.model_selection import train_test_split
X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=0.2)

from keras.utils.np_utils import to_categorical
y_train = to_categorical(y_train)
y_test = to_categorical(y_test)

# p.348 조기종료 설정 내용
from tensorflow.keras.callbacks import EarlyStopping
early_stopping = EarlyStopping(patience=2)

history = model.fit(X_train, y_train, validation_split=0.2, epochs=1000, callbacks=[early_stopping])


# In[4]: p.349
import matplotlib.pyplot as plt
history_dict = history.history
plt.clf()
acc = history_dict['accuracy']
epoch_range = range(1, len(acc)+1)
plt.plot(epoch_range, acc, 'b', label='Training acc')
plt.plot(epoch_range, history_dict['val_accuracy'], 'r:', label='Validation acc')
plt.axis([0, 1000, 0.0, 1.1])
plt.title('Trainging accuracy')
plt.xlabel('Epochs')
plt.ylabel('Accuracy')
plt.legend()
plt.show()


# In[5]:
plt.plot(epoch_range, history_dict['loss'], 'b', label='Trainingloss')
plt.plot(epoch_range, history_dict['val_loss'], 'b:', label='validation loss')
plt.title('Training and validation loss')
plt.axis([-0.3, 1000, 0, 2.0])
plt.xlabel('Epochs')
plt.ylabel('Loss')
plt.legend()
plt.show()


# In[6]: p.350
results = model.evaluate(X_test, y_test)


# In[7]:
from sklearn import datasets
iris = datasets.load_iris()
X = iris.data
y = iris.target
from sklearn.model_selection import train_test_split
X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=0.5)

from keras.utils.np_utils import to_categorical
y_train = to_categorical(y_train)
y_test = to_categorical(y_test)


# In[8]:
from tensorflow.keras import models
from tensorflow.keras.layers import Dense, Dropout
model = models.Sequential()
model.add(Dense(2, input_shape=(4, )))
model.add(Dense(16, activation='relu'))
model.add(Dense(32, activation='relu'))
model.add(Dense(3, activation='softmax'))

model.compile(optimizer='rmsprop', loss='categorical_crossentropy', metrics=['accuracy'])
epochs = 80
history = model.fit(X_train, y_train, validation_split=0.4, epochs=epochs)


# In[9]: p.351
import matplotlib.pyplot as plt
history_dict = history.history
plt.clf()
acc = history_dict['accuracy']
epoch_range = range(1, len(acc)+1)
plt.plot(epoch_range, acc, 'b', label='Training acc')
plt.plot(epoch_range, history_dict['val_accuracy'], 'r:', label='Validation acc')
plt.axis([0, epochs, 0.0, 1.1])
plt.title('Trainging accuracy')
plt.xlabel('Epochs')
plt.ylabel('Accuracy')
plt.legend()
plt.show()


# In[10]: p.352
plt.plot(epoch_range, history_dict['loss'], 'b', label='Trainingloss')
plt.plot(epoch_range, history_dict['val_loss'], 'b:', label='validation loss')
plt.title('Training and validation loss')
plt.axis([-0.3, epochs, 0, 2.0])
plt.xlabel('Epochs')
plt.ylabel('Loss')
plt.legend()
plt.show()


# In[11]:
from tensorflow.keras import models
from tensorflow.keras.layers import Dense, Dropout
model = models.Sequential()
model.add(Dense(2, input_shape=(4, )))
model.add(Dropout(0.2))
model.add(Dense(16, activation='relu'))
model.add(Dropout(0.2))
model.add(Dense(32, activation='relu'))
model.add(Dropout(0.2))
model.add(Dense(3, activation='softmax'))

